/* widgets.cpp

  Created by SMF aka Antoine Laydier (laydier@usa.net)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*=============================================================================
  HEADERs
 =============================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>


#include <qcombo.h>
#include <qpushbt.h>
#include <qtabdlg.h>
#include <qmsgbox.h>
#include <qlabel.h>
#include <qobject.h>
#include <qlistbox.h>
#include <qgrpbox.h>
#include <qradiobt.h>
#include <qbttngrp.h>
#include <qtabbar.h>
#include <qpopmenu.h>
#include <qdir.h>
#include <qlineedit.h>
#include <qchkbox.h>

#include <kconfig.h>
#include <kapp.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <ktablistbox.h>
#include <kurl.h>

#include "widgets.h"
#include "widgets.moc"

/*=============================================================================
  #DEFINEs
=============================================================================*/
#define ktr           klocale->translate
#define MP3FILTER     "*.mp[23]"

/*=============================================================================
  GLOBALs
=============================================================================*/
extern int      errno;

/*=============================================================================
 Class : TagEditor (methods)
=============================================================================*/

/*-----------------------------------------------------------------------------
  Routine : TagEditor::TagEditor (constructor)
            creates the actual QTabDialog. We create it as a modeless dialog, 
	    using the toplevel widget as its parent, so the dialog won't get 
	    its own window.
-----------------------------------------------------------------------------*/
TagEditor::TagEditor( QWidget *parent, const char *name, int sfolder,
		      QListBox * _list)
        :QTabDialog( parent, name, FALSE, 0)
{
  int i;
  fd = 0;
  layer =new Layer;
  tag = new Tag;
  fileList = _list;
  connect(fileList,SIGNAL(selected(int)),SLOT(open( void )));
   
  QString tmp;
  QWidget *p0,*p1;

  initMetaObject();

  pages[0] = NULL;
  pages[1] = NULL;

  setStyle(WindowsStyle);
    
  connect(tabBar(),SIGNAL(selected(int)),SLOT(tabBarSelected(int)));
     
  /*----------------------------------------------
    set up page 0 (MP3 Info viewer)
    ----------------------------------------------*/ 

  pages[0] = p0 = new QWidget(this,"page0"); 
  CHECK_PTR(p0);
  pInfo_box = new QGroupBox(p0, "pInfo_box"); 
  CHECK_PTR(pInfo_box);  

  pInfo_fileName = new QLabel(ktr("Filename"), p0);
  pInfo_fileName->move(18,40);  
  pInfo_fileName->setAutoResize(TRUE);  
  pInfo_fileName = new QLabel("",p0);
  pInfo_fileName->move(100,40);  
  pInfo_fileName->setAlignment(AlignRight);  

  pInfo_type = new QLabel(ktr("Type"), p0);
  pInfo_type->move(18,80);  
  pInfo_type->setAutoResize(TRUE);
  pInfo_type = new QLabel("",p0);
  pInfo_type->move(100,80);  
  pInfo_type->setAlignment(AlignRight);  

  pInfo_mode = new QLabel(ktr("Mode"), p0);
  pInfo_mode->move(18,120);  
  pInfo_mode->setAutoResize(TRUE);  
  pInfo_mode = new QLabel("",p0);
  pInfo_mode->move(100,120);  
  pInfo_mode->setAlignment(AlignRight);  

  pInfo_bitRate = new QLabel(ktr("BitRate"), p0);
  pInfo_bitRate->move(18,160);  
  pInfo_bitRate->setAutoResize(TRUE);  
  pInfo_bitRate = new QLabel("",p0);
  pInfo_bitRate->move(100,160);  
  pInfo_bitRate->setAlignment(AlignRight);  

  pInfo_sampleFreq = new QLabel(ktr("Frequency"), p0);
  pInfo_sampleFreq->move(18,200);  
  pInfo_sampleFreq->setAutoResize(TRUE);  
  pInfo_sampleFreq = new QLabel("",p0);
  pInfo_sampleFreq->move(100,200);  
  pInfo_sampleFreq->setAlignment(AlignRight);  

  pInfo_length = new QLabel(ktr("Length"), p0);
  pInfo_length->move(18,240);  
  pInfo_length->setAutoResize(TRUE);  
  pInfo_length = new QLabel("",p0);
  pInfo_length->move(100,240);  
  pInfo_length->setAlignment(AlignRight);  

  pInfo_box->setTitle(ktr("Informations"));
  
  /*----------------------------------------------
    set up page 1 (Tag Editor)
    ----------------------------------------------*/
  pages[1] = p1 = new QWidget(this, "page1"); 
  CHECK_PTR(p1);          

  pTag_box = new QGroupBox(p1,"pTag_box"); 
  CHECK_PTR(pTag_box); 
   
  pTag_CBtitle = new QCheckBox(ktr("Title"), p1);
  pTag_CBtitle->setGeometry(15,35,100,25);
  pTag_CBartist = new QCheckBox(ktr("Artist"), p1);
  pTag_CBartist->setGeometry(15,75,100,25); 
  pTag_CBalbum = new QCheckBox(ktr("Album"), p1);
  pTag_CBalbum->setGeometry(15,115,100,25);  
  pTag_CByear = new QCheckBox(ktr("Year"), p1);
  pTag_CByear->setGeometry(15,155,100,25);  
  pTag_CBgenre = new QCheckBox(ktr("Genre"), p1);
  pTag_CBgenre->setGeometry(15,195,100,25);  
  pTag_CBcomment = new QCheckBox(ktr("Comment"), p1);
  pTag_CBcomment->setGeometry(15,235,100,25);  

  pTag_fileName = new QLabel(ktr("Filename"), p1);
  pTag_fileName->move(15,275);  
  pTag_fileName->setAutoResize(TRUE);  

  pTag_leTitle = new QLineEdit(p1,"pTag_leTitle");
  pTag_leTitle->setGeometry(110,35,345,25); 
  pTag_leArtist = new QLineEdit(p1,"pTag_leArtist");
  pTag_leArtist->setGeometry(110,75,345,25); 
  pTag_leAlbum = new QLineEdit(p1,"pTag_leAlbum");
  pTag_leAlbum->setGeometry(110,115,345,25); 
  pTag_leYear = new QLineEdit(p1,"pTag_leYear");
  pTag_leYear->setGeometry(110,155,60,25); 
  pTag_cbGenre = new QComboBox(FALSE, p1, "pTag_cbGenre");
  pTag_cbGenre->setGeometry(110,195,200,25); 
  pTag_leComment = new QLineEdit(p1,"pTag_leComment");
  pTag_leComment->setGeometry(110,235,345,25); 
  pTag_fileName = new QLabel("",p1);
  pTag_fileName->move(110,275);  
  pTag_fileName->setAlignment(AlignRight);  

  for(i=0;i< (Tag::genre_largest+1);i++) 
    pTag_cbGenre->insertItem(Tag::genres[i],i);

  pTag_leTitle->setMaxLength(30);
  pTag_leArtist->setMaxLength(30);
  pTag_leAlbum->setMaxLength(30);
  pTag_leYear->setMaxLength(4);
  pTag_leComment->setMaxLength(30);

  pTag_box->setTitle(ktr("Tag Editor"));

  pTag_bSet = new QPushButton(ktr("&Set"), p1, "pTag_bSet");
  CHECK_PTR(pTag_bSet);
  connect(pTag_bSet,SIGNAL(clicked()), this, SLOT( pTag_bSetActivated()));
  pTag_bReset = new QPushButton(ktr("&Reset"), p1,"pTag_bReset");
  CHECK_PTR(pTag_bReset);
  connect(pTag_bReset,SIGNAL(clicked()), this,SLOT(pTag_bResetActivated()));

  initPannel();

  // startup_page settings...
  startup_page = PAGE_INFO;

  if ( sfolder >= 0 ) { 
    restoreStartupPage = TRUE;
    startup_page = sfolder;
  }

    // add pages...
  addTab(p0,ktr("&Informations"));
  addTab(p1,ktr("&Tag Editor"));
  move(0,0);
  
  // add buttons ...
  setOkButton(ktr("&Quit"));
  
  bOpen = new QPushButton(ktr("&Open"), this, "bOpen");
  CHECK_PTR(bOpen);
  connect(bOpen, SIGNAL(clicked()), this, SLOT(bOpenActivated()));
  bPrev = new QPushButton(ktr("&Previous"), this, "bPrev");
  CHECK_PTR(bPrev);
  connect(bPrev, SIGNAL(clicked()), this, SLOT(bPrevActivated()));
  bNext = new QPushButton(ktr("&Next"), this, "bNext");
  CHECK_PTR(bNext);
  connect(bNext, SIGNAL(clicked()), this, SLOT(bNextActivated()));
  bClear = new QPushButton(ktr("&Clear"), this, "bClear");
  CHECK_PTR(bClear);
  connect(bClear, SIGNAL(clicked()), this, SLOT(bClearActivated()));

  installEventFilter(this);
  placeWidgets();
}


/*-----------------------------------------------------------------------------
  Routine : TagEditor::~TagEditor() (destructor)
	    writes back config entries (current)
-----------------------------------------------------------------------------*/
TagEditor::~TagEditor()
{  
  fileList = 0;
  delete layer;
  delete tag;
}

/*-----------------------------------------------------------------------------
  Routine : TagEditor::raiseStartUpPage
-----------------------------------------------------------------------------*/
void TagEditor::raiseStartUpPage()
{ 
    tabBar()->setCurrentTab(startup_page);
} 

/*-----------------------------------------------------------------------------
  Routine : TagEditor::placeWidgets
-----------------------------------------------------------------------------*/
void TagEditor::placeWidgets( void )
{
  int w = pages[0]->width();
  int h = pages[0]->height();

  // Button Open, Prev, Next
  bOpen->setGeometry(w - 150, h +41, 75, 26);
  bPrev->setGeometry(10, h +41, 75, 26);
  bNext->setGeometry(85, h +41, 75, 26);
  bClear->setGeometry(170, h +41, 75, 26);
  
  // Informations
  pInfo_box->setGeometry(5, 5, w - 10, h - 20);
  pInfo_fileName->resize(w-10-110,20);
  pInfo_mode->resize(w-10-110,20);
  pInfo_type->resize(w-10-110,20);
  pInfo_bitRate->resize(w-10-110,20);
  pInfo_sampleFreq->resize(w-10-110,20); 
  pInfo_length->resize(w-10-110,20); 
  
  // Tag Editor
  pTag_box->setGeometry(5, 5, w - 10, h - 20);
  
  pTag_leTitle->resize(w-15-110,25);
  pTag_leArtist->resize(w-15-110,25);
  pTag_leAlbum->resize(w-15-110,25);
  pTag_leComment->resize(w-15-110,25);
  pTag_fileName->resize(w-15-110,20);

  pTag_bSet->setGeometry(w - 175, h - 50, 75, 26);
  pTag_bReset->setGeometry(w - 90, h - 50, 75, 26);
}


/*-----------------------------------------------------------------------------
  Routine : TagEditor::resizeEvent
-----------------------------------------------------------------------------*/
void TagEditor::resizeEvent(QResizeEvent *ev)
{
  QTabDialog::resizeEvent(ev);

  if( ! pages[1] || !pages[0] )
    return;

  placeWidgets();
}

/*-----------------------------------------------------------------------------
  Routine : TagEditor::tabBarSelected
-----------------------------------------------------------------------------*/
void TagEditor::tabBarSelected ( int tabIndx )
{ 
  switch ( tabIndx ) {
     case PAGE_INFO:
       break;
     case PAGE_TAG :
       break;
     default:
       break;
  }
}

/*-----------------------------------------------------------------------------
  Routine : TagEditor::closeFile(void)
-----------------------------------------------------------------------------*/
void TagEditor::closeFile()
{
 if(fd) fclose(fd);
 fd = 0;

}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::bOpenActivated
-----------------------------------------------------------------------------*/
void TagEditor::bOpenActivated( void )
{ 
 enqueue(0);
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::bNextActivated
-----------------------------------------------------------------------------*/
void TagEditor::bNextActivated( void )
{ 
  if(fileList->currentItem() < (int)(fileList->count()-1)) {
    fileList->setCurrentItem(fileList->currentItem()+1);
    open();
  }
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::bPrevActivated
-----------------------------------------------------------------------------*/
void TagEditor::bPrevActivated( void )
{ 
    if(fileList->currentItem() > 0) {
      fileList->setCurrentItem(fileList->currentItem()-1);
      open();
    }
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::bClearActivated
-----------------------------------------------------------------------------*/
void TagEditor::bClearActivated( void )
{ 
  if(fileList->count()>0) {
    closeFile();
    fileList->clear();
    initPannel();
  }
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::initPannel
-----------------------------------------------------------------------------*/
void TagEditor::initPannel( void )
{ 
  // Disable all items ...
  pTag_cbGenre->setEnabled(FALSE);
  pTag_leTitle->setEnabled(FALSE);
  pTag_leArtist->setEnabled(FALSE);
  pTag_leAlbum->setEnabled(FALSE);
  pTag_leYear->setEnabled(FALSE);
  pTag_leComment->setEnabled(FALSE);
  pTag_bSet->setEnabled(FALSE);
  pTag_bReset->setEnabled(FALSE);

  // put NULL value in all widgets ...
  pInfo_fileName->setText(ktr("<No file>")); 
  pInfo_type->setText("");
  pInfo_mode->setText("");
  pInfo_bitRate->setText("");
  pInfo_sampleFreq->setText("");
  pInfo_length->setText("");

  pTag_cbGenre->setCurrentItem(0);
  pTag_leTitle->setText("");
  pTag_leArtist->setText("");
  pTag_leAlbum->setText("");
  pTag_leYear->setText("");
  pTag_leComment->setText("");
  pTag_fileName->setText(ktr("<No file>")); 
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::pTag_bResetActivated
-----------------------------------------------------------------------------*/
void TagEditor::pTag_bResetActivated( void )
{ 
 pTag_Reset();
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::pTag_bSetActivated
-----------------------------------------------------------------------------*/
void TagEditor::pTag_bSetActivated( void )
{ 
 pTag_Set();
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::pTag_Reset
-----------------------------------------------------------------------------*/
void TagEditor::pTag_Reset( void )
{ 
 readTag(pTag_bSet->isEnabled());
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::pTag_Set
-----------------------------------------------------------------------------*/
int TagEditor::pTag_Set( void )
{ 
  tag->setTitle(pTag_leTitle->text());
  tag->setAlbum(pTag_leAlbum->text());
  tag->setArtist(pTag_leArtist->text());
  tag->setYear(pTag_leYear->text());
  tag->setComment(pTag_leComment->text());
  if(pTag_cbGenre->currentItem() < Tag::genre_largest)
    tag->setGenre(pTag_cbGenre->currentItem());
  else tag->setGenre(255);
  tag->set(fd);
  return 0;
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::processDir
-----------------------------------------------------------------------------*/
void TagEditor::processDir(char * dir) {
  QDir             *dir2proc;
  QStrList         *sltmp;
  unsigned int      i;
  char             *file;

  // Looking for MP3 files ...
  dir2proc = new QDir(dir, MP3FILTER);
  dir2proc->setFilter(QDir::All);  
  sltmp = (QStrList *)(dir2proc->entryList());
  for(i=0;i<sltmp->count();i++) {
    char * tmpfile = (i==0 ? sltmp->first()
		           : sltmp->next());
    file = new char[strlen(dir) +strlen(tmpfile) +1];
    strcpy(file, dir);
    strcat(file, tmpfile);
    enqueue(file);
    delete[] file;
  }
  delete dir2proc;

  // Looking for sub-directories ...
  dir2proc = new QDir(dir, "*");
  dir2proc->setFilter(QDir::Dirs);
  sltmp = (QStrList *)(dir2proc->entryList());
  for(i=0;i<sltmp->count();i++) {
    char * tmpfile = (i==0 ? sltmp->first()
		           : sltmp->next());
    if(strcmp(tmpfile,".") && strcmp(tmpfile,"..")) {
       file = new char[strlen(dir) +strlen(tmpfile) +2];
       strcpy(file, dir);
       strcat(file, tmpfile);
       strcat(file,"/");
       processDir(file);
       delete[] file;
    }
  }

  delete dir2proc;    

}


/*-----------------------------------------------------------------------------
  Routine :  TagEditor::enqueue
-----------------------------------------------------------------------------*/
int TagEditor::enqueue(char * fileName )
{ 
  QString           file(fileName);
  FILE             *fd2;
  unsigned int      i;
  struct stat       fileStat;

  // should I open the open dialog ?
  if(!fileName) {
    file = KFileDialog::getOpenFileName(0, MP3FILTER, this, 
					"Open");

    if(file.isNull())  return 0;
  }
  else file = fileName;
  KURL::decodeURL(file);

  // set Wait cursor ...
  QApplication::setOverrideCursor( waitCursor );

  // test if "file" is a directory ...
  if (stat (file.data(), &fileStat) || S_ISDIR(fileStat.st_mode)) {
    processDir(file.data());
    QApplication::restoreOverrideCursor(); 
    return 0;
  }

  // test if "file" is a MP3 file
  fd2 = 0;
  fd2 = fopen(file.data(), "r");
  if(!fd2) {
    QApplication::restoreOverrideCursor(); 
    QMessageBox::critical(this,
			  ktr("Error"), 
		ktr("The file specified cannot be enqueue\n\nSomething goes wrong !!!"));
    return 1;
  }
  if(layer->get(fd2) != true) {
    QApplication::restoreOverrideCursor(); 
    QMessageBox::critical(this,
			  ktr("Error"), 
			  ktr("The file specified \nis not a valid MP3 file\n\n"));
    fclose(fd2);
    return 1;
  }
  fclose(fd2);

  // is the file already in the list ?
  for(i=0;i<fileList->count();i++) 
    if(!strcmp(file.data(), fileList->text(i)))  {
       QApplication::restoreOverrideCursor(); 
       return 1;
    }
  // insert the file in list ...
  fileList->insertItem(file.data(), -1);
  if(fileList->count() ==1) { 
    fileList->setCurrentItem(0); 
    open(); 
  }
  
  QApplication::restoreOverrideCursor(); 
  return 0;
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::Open
-----------------------------------------------------------------------------*/
int TagEditor::open( void )
{ 
  char *  fileName;
  FILE *  fd2;
  bool    rw = TRUE;

  fileName = (char *)fileList->text(fileList->currentItem());

  if(!fileName) return 1; 

  fd2 = 0;
  fd2 = fopen(fileName, "r+");
  if(!fd2) {
    rw = FALSE;
    fd2 = fopen(fileName, "r");
    if(!fd2) {
      QMessageBox::critical(this,
			  ktr("Error"), 
	    ktr("The file specified cannot be open\n\nSomething goes wrong !!!"));
    
      return 1;
    }
  }
  if(layer->get(fd2) == true) {
    // read Informations and Tag ...
    char *dummy = new char[50]; 
    unsigned long length;
    int min, sec;
    closeFile();
    pInfo_fileName->setText( fileName ); 
    pTag_fileName->setText( fileName ); 
    fd = fd2;
    sprintf(dummy,"%s layer %s",layer->version_name(),layer->layer_name());
    pInfo_type->setText(dummy);
    pInfo_mode->setText(layer->mode_name());
    sprintf(dummy,"%d kbits/s",layer->bitrate());
    pInfo_bitRate->setText(dummy);
    sprintf(dummy,"%d Hz",layer->sfreq());
    pInfo_sampleFreq->setText(dummy);
    length = layer->length();
    min = (int)(length/60);
    sec = (int)(length - min*60);
    sprintf(dummy,"%d:%02d", min, sec );
    pInfo_length->setText(dummy);

    readTag(rw);
    pTag_bSet->setEnabled(rw);
    pTag_bReset->setEnabled(TRUE);
    delete[] dummy;
  }
  else {   
    QMessageBox::critical(this,
			  ktr("Error"), 
			  ktr("The file specified \nis not a valid MP3 file\n\n"));
    fclose(fd2);
  }
  return 0;
}

/*-----------------------------------------------------------------------------
  Routine :  TagEditor::readTag
-----------------------------------------------------------------------------*/
void  TagEditor::readTag ( bool rw )
{
  bool isTag = tag->get(fd);

  pTag_cbGenre->setEnabled(rw);
  if(!pTag_CBgenre->isChecked())
    if(isTag) 
      pTag_cbGenre->setCurrentItem(
	  tag->getGenreNum() >= Tag::genre_largest ? Tag::genre_largest 
				 		   : tag->getGenreNum() );
    else pTag_cbGenre->setCurrentItem(Tag::genre_largest);

  pTag_leTitle->setEnabled(TRUE);
  if(!pTag_CBtitle->isChecked())
    if(isTag) pTag_leTitle->setText(tag->getTitle());
    else pTag_leTitle->setText("");

  pTag_leArtist->setEnabled(TRUE);
  if(!pTag_CBartist->isChecked())
    if(isTag) pTag_leArtist->setText(tag->getArtist());
    else pTag_leArtist->setText("");

  pTag_leAlbum->setEnabled(TRUE);
  if(!pTag_CBalbum->isChecked())
    if(isTag) pTag_leAlbum->setText(tag->getAlbum());
    else pTag_leAlbum->setText("");

  pTag_leYear->setEnabled(TRUE);
  if(!pTag_CByear->isChecked())
    if(isTag) pTag_leYear->setText(tag->getYear());
    else pTag_leYear->setText("");

  pTag_leComment->setEnabled(TRUE);
  if(!pTag_CBcomment->isChecked())
    if(isTag) pTag_leComment->setText(tag->getComment());
    else pTag_leComment->setText("");
}
